@@ -10,7 +10,7 @@ require AutoLoader;
 
 use Carp;
 
-$VERSION = '0.7004';
+$VERSION = '0.7005';
 
 @ISA = qw(Exporter DynaLoader);
 # Items to export into callers namespace by default. Note: do not export
@@ -25,10 +25,26 @@ $VERSION = '0.7004';
 
 require Gnome::Types;
 
-bootstrap Gnome $VERSION;
-
 sub dl_load_flags {0x01}
 
+bootstrap Gnome $VERSION;
+
+sub getopt_options {
+	my $dummy;
+	return (
+		Gtk->getopt_options,
+		"disable-sound"	=> \$dummy,
+		"enable-sound"	=> \$dummy,
+		"espeaker=s"	=> \$dummy,
+		"version"	=> \$dummy,
+		"usage"	=> \$dummy,
+		"help|?"	=> \$dummy,
+		"sm-client-id=s"	=> \$dummy,
+		"sm-config-prefix=s"	=> \$dummy,
+		"sm-disable" => \$dummy,
+		"disable-crash-dialog" => \$dummy,
+		);
+}
 
 # Autoload methods go after __END__, and are processed by the autosplit program.
 
