@@ -9,7 +9,7 @@ require AutoLoader;
 
 use Carp;
 
-$VERSION = '0.7004';
+$VERSION = '0.7005';
 
 @ISA = qw(Exporter DynaLoader);
 # Items to export into callers namespace by default. Note: do not export
@@ -30,5 +30,9 @@ require Gtk::GLArea::Types;
 
 # Autoload methods go after __END__, and are processed by the autosplit program.
 
+Gtk->mod_init_add('Gtk', sub {
+	init Gtk::GLArea;
+});
+
 1;
 __END__
